import pandas as pd
from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class AVFAD(Base):
    DB_NAME = "avfad"
    __ignore_files = [
        "PLS007",  # 0 length audio
    ]

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        df = self.__read_data(source_path=source_path)
        return set(df["CMVD-I Dimension 1 (word system)"].tolist())

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        df = self.__read_data(source_path)
        for _, row in df.iterrows():
            speaker_id = row["File ID"]
            age = int(row["Age"])
            gender = Gender.format(row["Sex"].strip())
            diag_term = row["CMVD-I Dimension 1 (word system)"]
            diagnosis = (
                diagnosis_map[diag_term]
                if diag_term in diagnosis_map
                else diagnosis_map.unclassified
            )
            if allow_incomplete_classification or not diagnosis.incompletely_classified:
                file_paths = [
                    path
                    for path in Path(source_path).rglob(f"{speaker_id}*.wav")
                    if self.__include(path)
                ]
                num_files = len(file_paths)
                if min_tasks is None or num_files >= min_tasks:
                    sessions += [
                        ProcessedSession(
                            id=f"avfad_{speaker_id}",
                            speaker_id=speaker_id,
                            age=age,
                            gender=gender,
                            diagnosis=[diagnosis],
                            files=[ProcessedFile(path=path) for path in file_paths],
                            num_files=num_files,
                        )
                    ]

        return sessions

    def __read_data(self, source_path):
        df = pd.read_excel(f"{source_path}/AVFAD_01_00_00_1_README/AVFAD_01_00_00.xlsx")
        df = df[["File ID", "CMVD-I Dimension 1 (word system)", "Sex", "Age"]]
        df["CMVD-I Dimension 1 (word system)"] = df[
            "CMVD-I Dimension 1 (word system)"
        ].apply(self.__clean_diagnosis)
        return df

    def __clean_diagnosis(self, diagnosis: str) -> str:
        diagnosis = diagnosis.lower().strip()
        diagnosis = diagnosis.replace("–", "-")
        diagnosis = diagnosis.replace("’", "'")
        return diagnosis

    def __include(self, path: Path) -> bool:
        for exclusion in self.__ignore_files:
            if exclusion in str(path):
                return False
        return True



import asyncio
from pathlib import Path
from asyncio.subprocess import create_subprocess_exec
from .logger import Logger
from .prepare_dataset.processed import ProcessedFile


class Download:
    def __init__(self, database_path: Path, logger: Logger) -> None:
        self.__database_path = database_path
        self.__logger = logger

    async def all(self):
        await asyncio.gather(*[self.svd(), self.torgo(), self.voiced()])

    async def selected(self, datasets):
        await asyncio.gather(*[getattr(self, db)() for db in datasets])

    async def svd(self) -> None:
        db_key = "svd"
        db_path = Path(f"{self.__database_path}/{db_key}")
        db_path.mkdir(exist_ok=True, parents=True)
        proc = await create_subprocess_exec("python", "-m", "svd_downloader", db_path)
        await proc.wait()

    async def torgo(self) -> None:
        db_key = "torgo"
        torgo_path = Path(f"{self.__database_path}/{db_key}")
        torgo_path.mkdir(exist_ok=True, parents=True)
        base_link = "http://www.cs.toronto.edu/~complingweb/data/TORGO"

        async def download_and_extract(key):
            self.__logger.info(f"Downloading {db_key}/{key}")
            proc = await create_subprocess_exec(
                "wget", f"{base_link}/{key}", cwd=torgo_path
            )
            await proc.wait()
            self.__logger.info(f"Extracting {db_key}/{key}")
            proc = await create_subprocess_exec("tar", "-xf", key, cwd=torgo_path)
            await proc.wait()

        procs = map(
            download_and_extract, ["F.tar.bz2", "FC.tar.bz2", "M.tar.bz2", "MC.tar.bz2"]
        )
        await asyncio.gather(*procs)

    async def voiced(self) -> None:
        db_key = "voiced"
        voiced_path = Path(f"{self.__database_path}/{db_key}")
        voiced_path.mkdir(exist_ok=True, parents=True)
        zip_link = "https://physionet.org/static/published-projects/voiced/voiced-database-1.0.0.zip"
        proc = await create_subprocess_exec("wget", zip_link, cwd=voiced_path)
        await proc.wait()
        proc = await create_subprocess_exec(
            "unzip", "voiced-database-1.0.0.zip", cwd=voiced_path
        )
        await proc.wait()

        dat_files = list(Path(f"{voiced_path}").rglob("*.dat"))
        processes = []
        for file in dat_files:

            processes.append(
                ProcessedFile.from_wfdb(
                    dat_path=Path(str(file).replace(".dat", "")),
                    extraction_path=file.parent.resolve(),
                )
            )
        await asyncio.gather(*processes)



from pathlib import Path
from typing import List
from class_argparse import ClassArgParser

from .download import Download
from .logger import Logger
from .task_generator import VERSIONS, collect_diagnosis_terms, generate_tasks
from .data_counter import data_counter


class Main(ClassArgParser):
    def __init__(self) -> None:
        super().__init__(name="DiVR Benchmark")
        self.logger = Logger(log_path="/tmp/main.log", key="main")

    async def download_openaccess(
        self,
        database_path: Path,
        all: bool = False,
        datasets: List[str] = [],
    ) -> None:
        downloader = Download(database_path=database_path, logger=self.logger)
        if all:
            await downloader.all()
        elif len(datasets) != 0:
            await downloader.selected(datasets)
        else:
            print("Must specify either --all or --datasets")

    async def collect_diagnosis_terms(self, version: VERSIONS, data_store_path: Path):
        await collect_diagnosis_terms(version=version, source_path=data_store_path)

    async def count_per_db_and_diag_map(
        self, version: VERSIONS, data_store_path: Path, output_path: Path
    ):
        await data_counter(
            version=version,
            data_store_path=data_store_path,
            output_path=output_path,
        )


if __name__ == "__main__":
    main = Main()
    main()



import pandas as pd
from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from ..Base import Base
from ..gender import Gender
from ....prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class Torgo(Base):
    DB_NAME = "torgo"
    __ignore_files = [
        "FC01/Session1/wav_arrayMic/0256.wav",  # 0 length audio
    ]
    max_tasks = 142
    avg_tasks = 60
    min_tasks = 0
    __df = [
        {
            "id": "F01",
            "diagnosis": "without dysarthria",
            "gender": "F",
            "age": None,
        },
        {
            "id": "F03",
            "diagnosis": "without dysarthria",
            "gender": "F",
            "age": None,
        },
        {
            "id": "F04",
            "diagnosis": "without dysarthria",
            "gender": "F",
            "age": None,
        },
        {"id": "FC01", "diagnosis": "dysarthria", "gender": "F", "age": 28},
        {"id": "FC02", "diagnosis": "dysarthria", "gender": "F", "age": 24},
        {"id": "FC03", "diagnosis": "dysarthria", "gender": "F", "age": 21},
        {
            "id": "M01",
            "diagnosis": "without dysarthria",
            "gender": "M",
            "age": None,
        },
        {"id": "M02", "diagnosis": "without dysarthria", "gender": "M", "age": 57},
        {
            "id": "M03",
            "diagnosis": "without dysarthria",
            "gender": "M",
            "age": None,
        },
        {
            "id": "M04",
            "diagnosis": "without dysarthria",
            "gender": "M",
            "age": None,
        },
        {
            "id": "M05",
            "diagnosis": "without dysarthria",
            "gender": "M",
            "age": None,
        },
        {"id": "MC01", "diagnosis": "dysarthria", "gender": "M", "age": 20},
        {"id": "MC02", "diagnosis": "dysarthria", "gender": "M", "age": 26},
        {"id": "MC03", "diagnosis": "dysarthria", "gender": "M", "age": 29},
        {"id": "MC04", "diagnosis": "dysarthria", "gender": "M", "age": None},
    ]

    def __init__(
        self,
        source_path: Path,
    ) -> None:
        curdir = Path(__file__).parent.resolve()
        # only valid prompts that all speakers have spoken are selected
        self.__selected_prompts: List[str] = pd.read_csv(
            f"{curdir}/selected_prompts.csv"
        )["prompt"].to_list()

        super().__init__(
            source_path=source_path,
        )

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        return set([data["diagnosis"].lower() for data in self.__df])

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []

        for data in self.__df:
            speaker_id = data["id"]
            diagnosis = data["diagnosis"].lower()
            diagnosis = (
                diagnosis_map[diagnosis]
                if diagnosis in diagnosis_map
                else diagnosis_map.unclassified
            )
            speaker_path = Path(f"{source_path}/{speaker_id}")
            age = int(data["age"]) if data["age"] is not None else None
            gender = Gender.format(data["gender"])
            if not diagnosis.incompletely_classified or allow_incomplete_classification:
                for session in speaker_path.glob("Session*"):
                    files = self.__select_files(session=session)
                    num_files = len(files)
                    if min_tasks is None or num_files >= min_tasks:
                        sessions += [
                            ProcessedSession(
                                id=f"torgo_{speaker_id}_{session.name}",
                                speaker_id=speaker_id,
                                age=age,
                                gender=gender,
                                diagnosis=[diagnosis],
                                files=files,
                                num_files=num_files,
                            )
                        ]
        return sessions

    def __select_files(self, session):
        files = []
        for path in Path(f"{session}/wav_arrayMic").glob("*.wav"):
            if self.__include(path) and self.__is_allowed_prompt(path):
                files.append(ProcessedFile(path=path))
        return files

    def __include(self, path: Path):
        for exclusion in self.__ignore_files:
            if exclusion in str(path):
                return False
        return True

    def __is_allowed_prompt(self, wav_path: Path) -> bool:
        prompt_path = Path(
            str(wav_path).replace("wav_arrayMic", "prompts").replace(".wav", ".txt")
        )
        if not prompt_path.is_file():
            return False
        with open(prompt_path, "r") as prompt_file:
            line = prompt_file.readline()
            prompt = line.lower().replace('"', "").replace("'", "")
            return prompt in self.__selected_prompts



import numpy as np
import pandas as pd
from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class UncommonVoice(Base):
    DB_NAME = "uncommon_voice"

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        _, df = self.__read_data(source_path=source_path)
        return set(df["Voice Disorder"].tolist())

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        data_path, df = self.__read_data(source_path)
        for row_idx, data in df.iterrows():
            speaker_id = data["new_ID"]
            if speaker_id is None:
                speaker_id = str(row_idx)
            diagnosis = data["Voice Disorder"]
            diagnosis = (
                diagnosis_map[diagnosis]
                if diagnosis in diagnosis_map
                else diagnosis_map.unclassified
            )
            if allow_incomplete_classification or not diagnosis.incompletely_classified:
                age = None
                gender = Gender.format(data["Gender"].strip())
                file_paths = list(Path(data_path).glob(f"{speaker_id}_*.wav"))
                num_files = len(file_paths)
                if min_tasks is None or num_files >= min_tasks:
                    session = ProcessedSession(
                        id=f"uncommon_voice_{speaker_id}",
                        speaker_id=speaker_id,
                        age=age,
                        gender=gender,
                        diagnosis=[diagnosis],
                        files=[ProcessedFile(path=path) for path in file_paths],
                        num_files=num_files,
                    )
                    sessions += [session]
        return sessions

    def __read_data(self, source_path):
        data_path = f"{source_path}/UncommonVoice/UncommonVoice_final"
        df = pd.read_csv(f"{source_path}/uncommonvoice_user_data.csv").replace(
            {np.nan: None}
        )
        df = df[["new_ID", "Voice Disorder", "Gender"]]
        df["Voice Disorder"] = df["Voice Disorder"].apply(
            lambda x: "normal" if x == 0 else "pathological"
        )
        return data_path, df



import pandas as pd
from pathlib import Path


def collector(row):
    total_speakers = row["speaker_id"].count()
    min_sessions = row["session_id"].min()
    max_sessions = row["session_id"].max()
    average_sessions = row["session_id"].mean()
    data = {
        "total_speakers": total_speakers,
        "min_sessions": min_sessions,
        "max_sessions": max_sessions,
        "average_sessions": average_sessions,
    }
    return pd.Series(data)


location = "/home/divr_benchmark/storage/data/torgo/"
files = Path(location).rglob("prompts/*.txt")
all_prompts = []
for file_name in files:
    with open(file_name, "r") as file:
        line = file.readline().strip()
        prompt = line.lower().replace('"', "").replace("'", "")
    [speaker_id, session_id, _] = (
        str(file_name).removeprefix(location).split("/", maxsplit=2)
    )
    all_prompts.append(
        {
            "prompt": prompt,
            "speaker_id": speaker_id,
            "session_id": session_id,
        }
    )

curdir = Path(__file__).parent.resolve()
gdf = (
    pd.DataFrame(all_prompts)
    .groupby(["prompt", "speaker_id"])
    .count()
    .reset_index()
    .groupby("prompt")[["prompt", "speaker_id", "session_id"]]
    .apply(func=collector)
    .reset_index()
    .sort_values(by=["total_speakers", "prompt"], ascending=[False, True])
).to_csv(f"{curdir}/prompt_stats.csv", index=False)



from .benchmark import Benchmark, Result, Task, TrainPoint, TestPoint
from divr_diagnosis import Diagnosis, DiagnosisMap

__all__ = [
    "Benchmark",
    "Diagnosis",
    "DiagnosisMap",
    "Result",
    "Task",
    "TrainPoint",
    "TestPoint",
]



import itertools
import numpy as np
import pandas as pd
from typing import Dict, List, Tuple
from divr_diagnosis import Diagnosis


class Result:
    def __init__(self, data: List[Tuple[Diagnosis, Diagnosis]]) -> None:
        confusion = self.__create_confusion_matrix(data)
        for actual, predicted in data:
            confusion[predicted.name][actual.name] += 1
        self.confusion = (
            pd.DataFrame(confusion).fillna(0).sort_index(axis=0).sort_index(axis=1)
        )
        self.results = data

    @property
    def top_1_accuracy(self) -> float:
        confusion = self.confusion.to_numpy()
        total_per_class = np.maximum(1, confusion.sum(axis=1))
        corrects = confusion.diagonal()
        per_class_accuracy = corrects / total_per_class
        accuracy = per_class_accuracy.mean()
        return accuracy

    def __create_confusion_matrix(self, data):
        confusion: Dict[str, Dict[str, int]] = {}
        all_names = list(
            set(
                [actual.name for (actual, _) in data]
                + [predicted.name for (_, predicted) in data]
            )
        )
        product_name_pair = itertools.product(all_names, all_names)
        for actual, predicted in product_name_pair:
            if actual not in confusion:
                confusion[actual] = {}
            actual = confusion[actual]
            if predicted not in actual:
                actual[predicted] = 0
        return confusion

    # good metric here is an open research question

    # maybe a multi-level top-k accuracy score that penalizes
    # higher level classifications more than lower



from __future__ import annotations
from dataclasses import dataclass
from typing import Dict, List, Literal, Tuple

from .processed import ProcessedSession, ProcessedDataset


@dataclass
class Cup:
    occupancy: int
    sessions: List[ProcessedSession]

    def allocate_session(self, session: ProcessedSession) -> bool:
        if len(self.sessions) < self.occupancy:
            self.sessions.append(session)
            return True
        return False


@dataclass
class Bucket:
    capacity: int
    train: Cup
    test: Cup
    val: Cup

    @property
    def occupancy(self) -> int:
        return self.train.occupancy + self.test.occupancy + self.val.occupancy

    @property
    def has_space(self) -> bool:
        return self.capacity > self.occupancy

    @property
    def has_zeros(self) -> bool:
        return min([self.train.occupancy, self.test.occupancy, self.val.occupancy]) == 0

    def allocate_sessions(self, data: List[ProcessedSession]) -> List[str]:
        total = len(data)
        added = 0
        added_session_ids = []
        if total > 3:
            raise ValueError(
                "only 3 or less items should be added at a time to ensure every class appears in final datasets"
            )
        if (added < total) and self.test.allocate_session(data[added]):
            added_session_ids.append(data[added].id)
            added += 1
        if (added < total) and self.train.allocate_session(data[added]):
            added_session_ids.append(data[added].id)
            added += 1
        if (added < total) and self.val.allocate_session(data[added]):
            added_session_ids.append(data[added].id)
            added += 1
        return added_session_ids


class BucketCollection(Dict[str, Bucket]):
    def setup(
        self,
        total_train_len: int,
        total_test_len: int,
        total_val_len: int,
        train_split: float,
        test_split: float,
        val_split: float,
        values: List[Tuple[str, int]],
    ) -> BucketCollection:
        self.total_train_len = total_train_len
        self.total_test_len = total_test_len
        self.total_val_len = total_val_len
        self.train_split = train_split
        self.test_split = test_split
        self.val_split = val_split

        for diag_name, count in values:
            self.__add_diagnosis(diag_name, count)

        self.__fill_zeros()
        self.__fill_remaining()
        return self

    def to_dataset(self, db_name) -> ProcessedDataset:
        train_sessions = []
        test_sessions = []
        val_sessions = []
        for bucket in self.values():
            train_sessions += bucket.train.sessions
            test_sessions += bucket.test.sessions
            val_sessions += bucket.val.sessions
        return ProcessedDataset(
            db_name=db_name,
            train_sessions=train_sessions,
            test_sessions=test_sessions,
            val_sessions=val_sessions,
        )

    def __add_diagnosis(self, diag_name: str, count: int) -> None:
        train_len = int(self.train_split * count)
        val_len = int(self.val_split * count)
        test_len = count - train_len - val_len
        self[diag_name] = Bucket(
            capacity=count,
            train=Cup(occupancy=train_len, sessions=[]),
            test=Cup(occupancy=test_len, sessions=[]),
            val=Cup(occupancy=val_len, sessions=[]),
        )

    def __fill_zeros(self) -> None:
        for key, bucket in self.items():
            if bucket.has_space and bucket.has_zeros:
                (
                    current_train_len,
                    current_test_len,
                    current_val_len,
                ) = self.__count_sets()
                bucket = self[key]
                if (
                    bucket.test.occupancy == 0
                    and current_test_len < self.total_test_len
                    and bucket.has_space
                ):
                    bucket.test.occupancy = 1
                if (
                    bucket.train.occupancy == 0
                    and current_train_len < self.total_train_len
                    and bucket.has_space
                ):
                    bucket.train.occupancy = 1
                if (
                    bucket.val.occupancy == 0
                    and current_val_len < self.total_val_len
                    and bucket.has_space
                ):
                    bucket.val.occupancy = 1

    def __fill_remaining(self) -> None:
        while key := self.__has_remaining():
            (current_train_len, current_test_len, current_val_len) = self.__count_sets()
            bucket = self[key]
            keyed_counts = [
                [current_train_len - self.total_train_len, bucket.train],
                [current_test_len - self.total_test_len, bucket.test],
                [current_val_len - self.total_val_len, bucket.val],
            ]
            filtered_key_counts = filter(lambda x: x[0] < 0, keyed_counts)
            sorted_key_counts = sorted(
                filtered_key_counts,
                key=lambda x: x[0],
            )
            best_option = sorted_key_counts[0]
            best_option[1].occupancy += 1

    def __has_remaining(self) -> str | Literal[False]:
        for key, bucket in self.items():
            if bucket.has_space:
                return key
        return False

    def __count_sets(self) -> Tuple[int, int, int]:
        train_len = 0
        test_len = 0
        val_len = 0
        for bucket in self.values():
            train_len += bucket.train.occupancy
            test_len += bucket.test.occupancy
            val_len += bucket.val.occupancy
        return (train_len, test_len, val_len)



from __future__ import annotations
import wfdb
import nspfile
import soundfile
from pathlib import Path
from typing import List, Set
from dataclasses import dataclass
from divr_diagnosis import Diagnosis


@dataclass
class ProcessedFile:
    path: Path

    @property
    def __dict__(self):
        return {
            "path": str(self.path),
        }

    @staticmethod
    async def from_wfdb(dat_path: Path, extraction_path: Path) -> ProcessedFile:
        extracted_path = Path(f"{extraction_path}/{dat_path.name}.wav")
        record = wfdb.rdrecord(dat_path)
        sample_rate = record.fs
        audio = record.p_signal
        soundfile.write(extracted_path, audio, sample_rate)
        return ProcessedFile(path=extracted_path)

    @staticmethod
    async def from_nsp(nsp_path: Path, extraction_path: Path) -> ProcessedFile:
        extracted_path = Path(f"{extraction_path}/{nsp_path.name}.wav")
        sample_rate, audio = nspfile.read(nsp_path)
        soundfile.write(extracted_path, audio, sample_rate)
        return ProcessedFile(path=extracted_path)

    @staticmethod
    def from_json(json_data):
        return ProcessedFile(**json_data)


@dataclass
class ProcessedSession:
    id: str
    speaker_id: str
    age: int | None
    gender: str
    diagnosis: List[Diagnosis]
    files: List[ProcessedFile]
    num_files: int

    @property
    def __dict__(self):
        return {
            "id": self.id,
            "speaker_id": self.speaker_id,
            "age": self.age,
            "gender": self.gender,
            "diagnosis": [diagnosis.name for diagnosis in self.diagnosis],
            "files": self.files,
            "num_files": self.num_files,
        }

    @property
    def best_diagnosis(self) -> Diagnosis:
        sorted_diagnosis = sorted(self.diagnosis, reverse=True)
        complete_diagnosis = list(
            filter(lambda x: not x.incompletely_classified, sorted_diagnosis)
        )
        if len(complete_diagnosis) > 0:
            return complete_diagnosis[0]
        return sorted_diagnosis[0]

    def diagnosis_names_at_level(self, level: int) -> Set[str]:
        diag_names = set()
        for diagnosis in self.diagnosis:
            diag_names.add(diagnosis.at_level(level).name)
        return diag_names

    def diagnosis_at_level(self, level: int) -> List[Diagnosis]:
        diags = {}
        for diagnosis in self.diagnosis:
            diag = diagnosis.at_level(level)
            if diag.name not in diags:
                diags[diag.name] = diag
        return sorted(list(diags.values()), reverse=True)


@dataclass
class ProcessedDataset:
    db_name: str
    train_sessions: List[ProcessedSession]
    val_sessions: List[ProcessedSession]
    test_sessions: List[ProcessedSession]

    @property
    def __dict__(self):
        return {
            "db_name": self.db_name,
            "train_sessions": self.train_sessions,
            "val_sessions": self.val_sessions,
            "test_sessions": self.test_sessions,
        }

    @property
    def all_sessions(self) -> List[ProcessedSession]:
        return self.train_sessions + self.val_sessions + self.test_sessions



import yaml
import json

fp = "/home/workspace/benchmark/divr_benchmark/tasks/v1/streams/0/train.yml"

with open(fp, "r") as f:
    data = yaml.full_load(f)
    new_data = []
    for key, value in data.items():
        age = value["age"]
        audio_keys = value["audio_keys"]
        audio_key: str = audio_keys[0]
        gender = value["gender"]
        label = value["label"]
        if audio_key.endswith("a_n.wav"):
            new_data += [(age, gender, label, audio_key)]
    print(json.dumps(new_data).replace("[", "(").replace("]", ")"))



import logging
from pathlib import Path


class Logger:
    def __init__(self, log_path: str, key: str):
        root_logger = logging.getLogger()
        root_logger.setLevel(logging.INFO)
        __log_path = Path(f"{log_path}/{key}.log")
        __log_path.parent.mkdir(parents=True, exist_ok=True)
        formatter = logging.Formatter(
            "%(asctime)s:%(levelname)s : %(name)s : %(message)s"
        )
        file_handler = logging.FileHandler(__log_path)
        file_handler.setFormatter(formatter)
        root_logger.handlers.clear()
        root_logger.addHandler(file_handler)
        self.info = root_logger.info
        self.debug = root_logger.debug
        self.warn = root_logger.warn
        self.error = root_logger.error



from __future__ import annotations
import random
from typing import Dict, List, Tuple
from divr_diagnosis import Diagnosis

from .processed import ProcessedSession, ProcessedDataset
from .database_plan import BucketCollection


class DatabaseGenerator:
    """
    This class takes a best effort approach to put a proportional amount
    of diagnosis, age and gender in the three different datasets i.e. train, val and test.
    While also making sure that any given pathology appears in each train, test and val sets
    at least once.

    First priority goes to diagnosis, then gender and then age.
    Age is considered in buckets of 10 i.e. 0-10, 11-20, 21-30, ...

    As we don't have a lot of diagnosis at every level of diagnosis, if a diagnosis can not
    be distributed equitably at a given level it would be resolved later at a parent level
    along with other unresolved diagnosis.

    Since a diagnosis can have multiple parents, the parent that it gets grouped with for
    distribution is decided with majority vote across the parent weight. In case of a tie
    classes are chosen in the order of pathological, normal and then unclassified.

    Since a session can have multiple diagnosis, and we don't have a diagnostic confidence
    metric as of now, the diagnosis that best balances out the dataset is chosen. This is
    achieved by chosing the diagnosis with most occurences in the input sessions so that
    it can appear in all train, test and val sets.

    We only consider balance of the dataset in terms of sessions and not in terms of files,
    this can result in slight imbalances in data if more sessions are recorded for a given
    pathology.
    """

    def __init__(
        self,
        train_split: float,
        test_split: float,
        random_seed: float,
    ) -> None:
        self.train_split = train_split
        self.test_split = test_split
        self.val_split = 1 - train_split - test_split
        self.random_seed = random_seed

    def generate(
        self,
        db_name: str,
        sessions: List[ProcessedSession],
    ) -> ProcessedDataset:
        random.Random(self.random_seed).shuffle(sessions)

        all_levels = []
        for session in sessions:
            for diag in session.diagnosis:
                all_levels.append(diag.level)
        max_level = max(all_levels)
        sorted_diag_counts = self.__sort_at_level(sessions, max_level)
        total_count = sum([v["count"] for v in sorted_diag_counts.values()])
        total_train_len = int(self.train_split * total_count)
        total_val_len = int(self.val_split * total_count)
        total_test_len = total_count - total_train_len - total_val_len
        level_diag_counts = self.__to_specific_level(
            sorted_diag_counts=sorted_diag_counts,
            level=0,
        )
        bucket_values = [
            (v["diag"].name, v["count"]) for v in level_diag_counts.values()
        ]
        weighted_buckets = BucketCollection().setup(
            total_train_len=total_train_len,
            total_test_len=total_test_len,
            total_val_len=total_val_len,
            train_split=self.train_split,
            test_split=self.test_split,
            val_split=self.val_split,
            values=bucket_values,
        )
        for level in range(1, max_level + 1):
            level_diag_counts = self.__to_specific_level(
                sorted_diag_counts=sorted_diag_counts,
                level=level,
            )
            new_buckets = BucketCollection()
            for key, bucket in weighted_buckets.items():
                filtered_counts = filter(
                    lambda x: x["diag"].satisfies(key), level_diag_counts.values()
                )
                bucket_values = [(v["diag"].name, v["count"]) for v in filtered_counts]
                new_bucket = BucketCollection().setup(
                    total_train_len=bucket.train.occupancy,
                    total_test_len=bucket.test.occupancy,
                    total_val_len=bucket.val.occupancy,
                    train_split=self.train_split,
                    test_split=self.test_split,
                    val_split=self.val_split,
                    values=bucket_values,
                )
                new_buckets.update(new_bucket)
            weighted_buckets = new_buckets

        while len(sorted_diag_counts) > 0:
            for diag_name, diag in sorted_diag_counts.items():
                selected_sessions = self.__select_gender_and_age(diag["sessions"])
                bucket = weighted_buckets[diag_name]
                sessions_added = bucket.allocate_sessions(selected_sessions[:3])
                for session_id in sessions_added:
                    del diag["sessions"][session_id]
                    diag["count"] -= 1
            for diag_name, max_diag in list(sorted_diag_counts.items()):
                if max_diag["count"] == 0:
                    del sorted_diag_counts[diag_name]

        return weighted_buckets.to_dataset(db_name)

    def __to_specific_level(self, sorted_diag_counts: Dict, level: int):
        counts: Dict = {}
        for val in sorted_diag_counts.values():
            count = val["count"]
            diag = val["diag"].at_level(level)
            if diag.name not in counts:
                counts[diag.name] = {
                    "count": count,
                    "diag": diag,
                }
            else:
                counts[diag.name]["count"] += count
        sorted_counts = dict(
            sorted(
                counts.items(),
                key=lambda x: x[1]["count"],
                reverse=True,
            )
        )
        return sorted_counts

    def __sort_at_level(self, sessions: List[ProcessedSession], level: int):
        working_sessions = sessions.copy()
        counts = {}
        while len(working_sessions) > 0:
            best_diag = self.__most_popular_diag(working_sessions, level)
            if best_diag.name not in counts:
                counts[best_diag.name] = {"diag": best_diag, "count": 0, "sessions": {}}
            for session in working_sessions:
                if best_diag.name in session.diagnosis_names_at_level(level):
                    counts[best_diag.name]["count"] += 1
                    counts[best_diag.name]["sessions"][session.id] = session
                    working_sessions.remove(session)
        sorted_counts = dict(
            sorted(
                counts.items(),
                key=lambda x: x[1]["count"],
                reverse=True,
            )
        )
        return sorted_counts

    def __most_popular_diag(
        self, sessions: List[ProcessedSession], level: int
    ) -> Diagnosis:
        best_diag = {}
        for session in sessions:
            for diag in session.diagnosis_at_level(level):
                if diag.name not in best_diag:
                    best_diag[diag.name] = {"diag": diag, "count": 1}
                else:
                    best_diag[diag.name]["count"] += 1
        return max(best_diag.items(), key=lambda x: x[1]["count"])[1]["diag"]

    def __select_gender_and_age(
        self, sessions: Dict[str, ProcessedSession]
    ) -> List[ProcessedSession]:
        grouped_sessions = {}
        for session in sessions.values():
            gender = session.gender
            age_bracket = self.__age_to_bracket(session.age)
            key = (gender, age_bracket)
            if key not in grouped_sessions:
                grouped_sessions[key] = []
            grouped_sessions[key].append(session)

        sorted_sessions_by_count = sorted(
            grouped_sessions.values(), key=lambda x: len(x), reverse=True
        )

        selected_sessions = []
        for sorted_sessions in sorted_sessions_by_count:
            selected_sessions += sorted_sessions
            if len(selected_sessions) > 2:
                return selected_sessions
        return selected_sessions

    def __age_to_bracket(self, age: int | None) -> Tuple[int, int]:
        if age is None:
            return (-1, 0)
        lower = (age // 10) * 10
        upper = lower + 10
        return (lower, upper)



import yaml
import numpy as np
from tqdm import tqdm
from pathlib import Path
from dataclasses import dataclass
from typing import Dict, List, Tuple
from divr_diagnosis import Diagnosis, DiagnosisMap

from .result import Result
from .audio_loader import AudioLoader


@dataclass
class TrainPoint:
    audio: List[np.ndarray]
    label: Diagnosis


@dataclass
class TestPoint:
    id: str
    audio: List[np.ndarray]


@dataclass
class DataPoint:
    id: str
    audio: List[np.ndarray]
    label: Diagnosis

    def to_testpoint(self) -> TestPoint:
        return TestPoint(
            id=self.id,
            audio=self.audio,
        )

    def to_trainpoint(self) -> TrainPoint:
        return TrainPoint(
            audio=self.audio,
            label=self.label,
        )

    def satisfies(self, prediction: str) -> bool:
        return self.label.satisfies(prediction)


class Task:

    __train: List[DataPoint]
    __val: List[DataPoint]
    __test: Dict[str, DataPoint]

    def __init__(
        self,
        diagnosis_map: DiagnosisMap,
        audio_loader: AudioLoader,
        train: Path,
        val: Path,
        test: Path,
        quiet: bool,
        diag_level: int | None,
        load_audios: bool,
    ) -> None:
        self.__diagnosis_map = diagnosis_map
        self.__audio_loader = audio_loader
        self.__train = self.__load_file(
            data_file=train,
            key="train",
            quiet=quiet,
            diag_level=diag_level,
            load_audios=load_audios,
        )
        self.__val = self.__load_file(
            data_file=val,
            key="val",
            quiet=quiet,
            diag_level=diag_level,
            load_audios=load_audios,
        )
        self.__test = dict(
            [
                (v.id, v)
                for v in self.__load_file(
                    data_file=test,
                    key="test",
                    quiet=quiet,
                    diag_level=diag_level,
                    load_audios=load_audios,
                )
            ]
        )
        self.audio_sample_rate = audio_loader.sample_rate
        self.__diagnosis_indices = self.__count_diagnosis()
        self.__max_diag_level = list(self.__diagnosis_indices.keys())[-1]

    @property
    def max_diag_level(self) -> int:
        return self.__max_diag_level

    @property
    def sample_rate(self) -> int:
        return self.__audio_loader.sample_rate

    def unique_diagnosis(self, level: int | None = None) -> List[str]:
        if level is None:
            level = self.__max_diag_level
        _, diag_indices_reversed = self.__diagnosis_indices[level]
        return list(diag_indices_reversed.keys())

    def index_to_diag(self, index: int, level: int | None = None) -> Diagnosis:
        if level is None:
            level = self.__max_diag_level
        diag_indices, _ = self.__diagnosis_indices[level]
        return diag_indices[index]

    def diag_to_index(self, diag: Diagnosis, level: int | None = None) -> int:
        if level is None:
            level = self.__max_diag_level
        _, diag_indices_reversed = self.__diagnosis_indices[level]
        return diag_indices_reversed[diag.at_level(level).name]

    def diag_name_to_index(self, diag_name: str, level: int | None = None) -> int:
        if level is None:
            level = self.__max_diag_level
        _, diag_indices_reversed = self.__diagnosis_indices[level]
        diag = self.__diagnosis_map.get(name=diag_name)
        return diag_indices_reversed[diag.at_level(level).name]

    def train_class_weights(self, level: int | None = None) -> List[float]:
        """
        Returns a list of (total_samples / class_samples)
        """
        if level is None:
            level = self.__max_diag_level
        diags = np.array([t.label.at_level(level).name for t in self.__train])
        class_weights = [0.0] * len(self.unique_diagnosis(level=level))
        _, diag_indices_reversed = self.__diagnosis_indices[level]
        for diag_name, count in zip(*np.unique(diags, return_counts=True)):
            idx = diag_indices_reversed[diag_name]
            class_weights[idx] = count
        total_sessions = sum(class_weights)
        for i in range(len(class_weights)):
            class_weights[i] = total_sessions / class_weights[i]
        return class_weights

    @property
    def train(self) -> List[DataPoint]:
        return self.__train

    @property
    def val(self) -> List[DataPoint]:
        return self.__val

    @property
    def test(self) -> List[DataPoint]:
        return list(self.__test.values())

    def test_label(self, id: str) -> Diagnosis:
        return self.__test[id].label

    def score(self, predictions: Dict[str, int]) -> Result:
        """
        predictions: Dict[test_id, index_of_diag]
        """
        results: List[Tuple[Diagnosis, Diagnosis]] = []
        for test_id, predicted_diagnosis in predictions.items():
            predicted_diagnosis = self.index_to_diag(predicted_diagnosis)
            actual_diagnosis = self.__test[test_id].label
            results += [(actual_diagnosis, predicted_diagnosis)]
        return Result(data=results)

    def __load_file(
        self,
        data_file: Path,
        key: str,
        quiet: bool,
        diag_level: int | None,
        load_audios: bool,
    ) -> List[DataPoint]:
        with open(data_file, "r") as df:
            data = yaml.load(df, Loader=yaml.FullLoader)
        dataset: List[DataPoint] = []
        if not quiet:
            iterator = tqdm(data.items(), desc=f"Loading {key} files", leave=False)
        else:
            iterator = data.items()
        for key, val in iterator:
            label = self.__diagnosis_map.get(val["label"])
            if diag_level is not None:
                label = label.at_level(diag_level)
            if load_audios:
                audio = self.__audio_loader(val["audio_keys"])
                if len(audio) < 1:
                    raise ValueError(f"Invalid data point (no audio): {key}")
            else:
                audio = []
            dataset.append(DataPoint(id=key, audio=audio, label=label))
        return dataset

    def __count_diagnosis(
        self,
    ) -> Dict[int, Tuple[Dict[int, Diagnosis], Dict[str, int]]]:
        train_diags = [d.label for d in self.__train]
        test_diags = [d.label for d in self.__test.values()]
        val_diags = [d.label for d in self.__val]
        unique_diagnosis = set(train_diags + test_diags + val_diags)
        max_level = max([d.level for d in unique_diagnosis])
        counts = {}
        for level in range(max_level + 1):
            diags_at_level = set([d.at_level(level) for d in unique_diagnosis])
            counts[level] = [
                dict(enumerate(sorted(diags_at_level, key=lambda x: x.name))),
                dict(
                    [
                        (diag.name, idx)
                        for idx, diag in enumerate(
                            sorted(diags_at_level, key=lambda x: x.name)
                        )
                    ]
                ),
            ]
        return counts



import json
from pathlib import Path
from typing import List, Literal, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


VOWELS = Literal["a", "i", "u", ""]


class SVD(Base):
    DB_NAME = "svd"
    ignore_files = [
        "1405/713/713-iau.wav",  # invalid file
        "1405/713/713-i_n.wav",  # invalid file
    ]
    max_tasks = 14

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        diags = set()
        with open(f"{source_path}/data.json", "r") as inputfile:
            for speaker_id, val in json.loads(inputfile.read()).items():
                for session in val["sessions"]:
                    classification = session["classification"]
                    pathologies = session["pathologies"]
                    diagnosis = pathologies if pathologies != "" else classification
                    for x in diagnosis.split(","):
                        diags.add(x.strip().lower())
        return diags

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        with open(f"{source_path}/data.json", "r") as inputfile:
            for speaker_id, val in json.loads(inputfile.read()).items():
                gender = Gender.format(val["gender"])
                for session in val["sessions"]:
                    session = self.__process_session(
                        speaker_id=speaker_id,
                        gender=gender,
                        source_path=source_path,
                        session=session,
                        allow_incomplete_classification=allow_incomplete_classification,
                        diagnosis_map=diagnosis_map,
                    )
                    if (session is not None) and (
                        min_tasks is None or session.num_files >= min_tasks
                    ):
                        sessions += [session]
        return sessions

    def __process_session(
        self,
        speaker_id,
        gender,
        source_path,
        session,
        allow_incomplete_classification: bool,
        diagnosis_map: DiagnosisMap,
    ):
        session_id = session["session_id"]
        age = int(session["age"])
        classification = session["classification"]
        pathologies = session["pathologies"]
        files = session["files"]
        input_diagnosis = pathologies if pathologies != "" else classification
        files = []
        for file in session["files"]:
            path = Path(
                f"{source_path}/{classification}/{gender}/{speaker_id}/{session_id}/{file.split('file=')[1]}.wav"
            )
            if self.__include(path):
                files += [ProcessedFile(path=path)]
        if len(files) == 0:
            return None
        diagnosis = []
        for x in input_diagnosis.split(","):
            x = x.strip().lower()
            if x in diagnosis_map:
                diagnosis += [diagnosis_map[x]]
            else:
                diagnosis += [diagnosis_map.unclassified]
        session = ProcessedSession(
            id=f"svd_{speaker_id}_{session_id}",
            speaker_id=speaker_id,
            age=age,
            gender=gender,
            diagnosis=diagnosis,
            files=files,
            num_files=len(files),
        )
        if (
            session.best_diagnosis.incompletely_classified
            and not allow_incomplete_classification
        ):
            if session.id == "svd_2537_2403":
                print(session.best_diagnosis.name)
                # print([d.name for d in sorted_diagnosis])
                # print([d.name for d in complete_diagnosis])
                # print(complete_diagnosis[0].incompletely_classified)
                exit()
            return None
        return session

    def __include(self, path: Path):
        for exclusion in self.ignore_files:
            if exclusion in str(path):
                return False
        return True

    def train_set_multi_neutral_vowels(self, level: int, vowels: List[VOWELS]):
        return self.filtered_multi_file_tasks(
            self.dataset.train_sessions,
            level=level,
            suffixes=[f"{v}_n.wav" for v in vowels],
        )

    def val_set_multi_neutral_vowels(self, level: int, vowels: List[VOWELS]):
        return self.filtered_multi_file_tasks(
            self.dataset.val_sessions,
            level=level,
            suffixes=[f"{v}_n.wav" for v in vowels],
        )

    def test_set_multi_neutral_vowels(self, level: int, vowels: List[VOWELS]):
        return self.filtered_multi_file_tasks(
            self.dataset.test_sessions,
            level=level,
            suffixes=[f"{v}_n.wav" for v in vowels],
        )

    def train_set_combined_vowel_vocalisation(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.train_sessions, level=level, suffix="iau.wav"
        )

    def val_set_combined_vowel_vocalisation(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.val_sessions, level=level, suffix="iau.wav"
        )

    def test_set_combined_vowel_vocalisation(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.test_sessions, level=level, suffix="iau.wav"
        )

    def train_set_lhl_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.train_sessions, level=level, suffix=f"{vowel}_lhl.wav"
        )

    def val_set_lhl_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.val_sessions, level=level, suffix=f"{vowel}_lhl.wav"
        )

    def test_set_lhl_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.test_sessions, level=level, suffix=f"{vowel}_lhl.wav"
        )

    def train_set_neutral_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.train_sessions, level=level, suffix=f"{vowel}_n.wav"
        )

    def val_set_neutral_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.val_sessions, level=level, suffix=f"{vowel}_n.wav"
        )

    def test_set_neutral_vowels(self, level: int, vowel: VOWELS = ""):
        return self.filtered_single_file_tasks(
            self.dataset.test_sessions, level=level, suffix=f"{vowel}_n.wav"
        )

    def train_set_connected_speech(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.train_sessions, level=level, suffix="-phrase.wav"
        )

    def val_set_connected_speech(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.val_sessions, level=level, suffix="-phrase.wav"
        )

    def test_set_connected_speech(self, level: int):
        return self.filtered_single_file_tasks(
            self.dataset.test_sessions, level=level, suffix="-phrase.wav"
        )

    def filtered_single_file_tasks(
        self, sessions: List[ProcessedSession], level: int, suffix: str
    ):
        def filter_func(files: List[ProcessedFile]) -> List[ProcessedFile]:
            return list(filter(lambda x: str(x.path).endswith(suffix), files))

        return self.to_individual_file_tasks(
            sessions, level=level, file_filter=filter_func
        )

    def filtered_multi_file_tasks(
        self, sessions: List[ProcessedSession], level: int, suffixes: List[str]
    ):
        def file_name_filter_func(processed_file: ProcessedFile):
            file_path_str = str(processed_file.path)
            for suffix in suffixes:
                if file_path_str.endswith(suffix):
                    return True
            return False

        def filter_func(files: List[ProcessedFile]) -> List[ProcessedFile]:
            return list(filter(file_name_filter_func, files))

        return self.to_multi_file_tasks(sessions, level=level, file_filter=filter_func)



import yaml
import statistics
from pathlib import Path
from dataclasses import dataclass
from typing import List

from .task import Task
from .databases import Base as Database
from typing import Protocol


class DatabaseFunc(Protocol):
    async def __call__(self, name: str, min_tasks: int | None = None) -> Database:
        ...


@dataclass
class Dataset:
    train: List[Task]
    val: List[Task]
    test: List[Task]


class Generator:
    def to_task_file(self, tasks: List[Task], output_path: Path) -> None:
        tasks_dict = {}
        for task in tasks:
            if len(task.audio_keys) < 1:
                raise ValueError(f"Invalid task (no audio keys): {task.id}")
            task_data = task.__dict__.copy()
            del task_data["id"]
            task_data["label"] = task.label.name
            tasks_dict[task.id] = task_data
        with open(f"{output_path}.yml", "w") as output_file:
            yaml.dump(tasks_dict, output_file)
        self.generate_demographics(tasks=tasks, output_path=output_path)

    def generate_demographics(self, tasks: List[Task], output_path: Path) -> None:
        demographics = {}
        for task in tasks:
            diagnosis_name = task.label.name
            if diagnosis_name not in demographics:
                demographics[diagnosis_name] = {}
            diagnosis = demographics[diagnosis_name]
            gender = task.gender
            if gender not in diagnosis:
                diagnosis[gender] = {"ages": [], "total": 0}
            diagnosis[gender]["total"] += 1
            if task.age is not None:
                diagnosis[gender]["ages"] += [task.age]
        for diagnosis in demographics:
            for gender in demographics[diagnosis]:
                ages = demographics[diagnosis][gender]["ages"]
                total = demographics[diagnosis][gender]["total"]
                total_ages = len(ages)
                age_stats = None
                if total_ages > 0:
                    age_stats = {
                        "mean": statistics.mean(ages),
                        "std": statistics.stdev(ages) if total_ages > 1 else None,
                        "min": min(ages),
                        "max": max(ages),
                    }

                demographics[diagnosis][gender] = {
                    "total": total,
                    "age_stats": age_stats,
                }
        with open(f"{output_path}.demographics.yml", "w") as output_file:
            yaml.dump(demographics, output_file)

    def truncate_low_resource_classes(
        self, task_list: List[List[Task]], min_examples: int
    ) -> List[List[Task]]:
        to_remove = []
        for tasks in task_list:
            counts = {}
            for task in tasks:
                diagnosis_name = task.label.name
                if diagnosis_name not in counts:
                    counts[diagnosis_name] = 0
                counts[diagnosis_name] += 1
            for label, count in counts.items():
                if count < min_examples:
                    to_remove += [label]
        new_list = []
        for tasks in task_list:
            new_tasks = []
            for task in tasks:
                diagnosis_name = task.label.name
                if diagnosis_name not in to_remove:
                    new_tasks += [task]
            new_list += [new_tasks]
        return new_list



val_map = {
    "f": "female",
    "m": "male",
    "female": "female",
    "male": "male",
    "unknown": "unknown",
    "": "unknown",
}


class Gender(str):
    @staticmethod
    def format(value: str) -> str:
        return val_map[value.strip().lower()]



import yaml
import typing
import asyncio
from pathlib import Path
from typing import Literal
from divr_diagnosis import DiagnosisMap

from .audio_loader import AudioLoader
from .task import Task
from ..logger import Logger
from ..download import Download
from ..task_generator import generator_map

VERSIONS = Literal["v1"]
versions = typing.get_args(VERSIONS)
task_generator_maps = {"v1": generator_map["v1"]}


class Benchmark:
    def __init__(
        self,
        storage_path: str | Path,
        version: VERSIONS,
        quiet: bool = False,
        sample_rate: int = 16000,
    ) -> None:
        self.__quiet = quiet
        if not Path(storage_path).is_dir():
            raise ValueError(
                f"storage_path: ({storage_path}) is not a valid directory."
            )
        if version not in versions:
            raise ValueError(
                f"invalid version ({version}) selected. Choose from: {versions}"
            )
        module_path = Path(__file__).parent.parent.resolve()
        self.__logger = Logger(log_path=f"{storage_path}/logs", key=f"{version}")
        self.__data_path = Path(f"{storage_path}/data")
        self.__audio_loader = AudioLoader(version, self.__data_path, sample_rate)
        self.__downloader = Download(
            database_path=self.__data_path, logger=self.__logger
        )
        self.__tasks_path = f"{module_path}/tasks/{version}"
        self.__task_generator = task_generator_maps[version]
        self.__ensure_datasets(tasks_path=self.__tasks_path)

    async def generate_task(
        self,
        filter_func,
        task_path: Path,
        diagnosis_map: DiagnosisMap,
        allow_incomplete_classification: bool,
    ) -> None:
        await self.__task_generator.generate_task(
            source_path=self.__data_path,
            filter_func=filter_func,
            task_path=task_path,
            diagnosis_map=diagnosis_map,
            allow_incomplete_classification=allow_incomplete_classification,
        )

    def load_task(
        self,
        task_path: Path,
        diag_level: int | None,
        diagnosis_map: DiagnosisMap,
        load_audios: bool = True,
    ) -> Task:
        if not task_path.is_dir():
            raise ValueError("Invalid task selected")
        return Task(
            diagnosis_map=diagnosis_map,
            audio_loader=self.__audio_loader,
            train=Path(f"{task_path}/train.yml"),
            val=Path(f"{task_path}/val.yml"),
            test=Path(f"{task_path}/test.yml"),
            quiet=self.__quiet,
            diag_level=diag_level,
            load_audios=load_audios,
        )

    def __ensure_datasets(self, tasks_path: str) -> None:
        datasets_file = f"{tasks_path}/datasets.yml"
        with open(datasets_file, "r") as df:
            datasets = yaml.load(df, Loader=yaml.FullLoader)

        to_download = []
        for dataset in datasets:
            dataset_path = Path(f"{self.__data_path}/{dataset}")
            if dataset_path.is_dir():
                self.__logger.info(f"{dataset} already exists at {dataset_path}")
            else:
                self.__logger.info(
                    f"{dataset} does not exist. Will create at {dataset_path}"
                )
                to_download.append(dataset)
        coro = self.__downloader.selected(datasets=to_download)
        try:
            loop = asyncio.get_running_loop()
            asyncio.run_coroutine_threadsafe(coro=coro, loop=loop)
        except RuntimeError:
            asyncio.run(coro)



from typing import List
import librosa
import numpy as np
from pathlib import Path


class AudioLoader:
    def __init__(self, version: str, data_path: Path, sample_rate: int) -> None:
        self.__data_path = data_path
        self.__version = version
        self.sample_rate = sample_rate

    def __call__(self, keys: List[str]) -> List[np.ndarray]:
        if self.__version == "v1":
            return list(map(self.__v1, keys))
        raise ValueError(f"invalid version {self.__version} selected in AudioLoader")

    def __v1(self, key: str) -> np.ndarray:
        audio_path = f"{self.__data_path}/{key}"
        audio, _ = librosa.load(audio_path, sr=self.sample_rate)
        if len(audio) < 1:
            raise ValueError(f"0 length audio file: {audio_path}")
        return self.__normalize(audio)

    def __normalize(self, audio: np.ndarray) -> np.ndarray:
        mean = np.mean(audio, axis=0)
        stddev = np.std(audio, axis=0)
        normalized_audio = (audio - mean) / stddev
        return normalized_audio



from tqdm import tqdm
from pathlib import Path
from divr_diagnosis import diagnosis_maps

from .task_generator import generator_map, VERSIONS


async def data_counter(
    version: VERSIONS,
    data_store_path: Path,
    output_path: Path,
) -> None:
    generator = generator_map[version]
    data_store_path = data_store_path
    db_names = [
        "avfad",
        "meei",
        "svd",
        "torgo",
        "uaspeech",
        "uncommon_voice",
        "voiced",
    ]
    diag_maps = [
        diagnosis_maps.Benba_2017,
        diagnosis_maps.Compton_2022,
        diagnosis_maps.Cordeiro_2015,
        diagnosis_maps.daSilvaMoura_2024,
        diagnosis_maps.deMoraesLimaMarinus_2013,
        diagnosis_maps.FEMH_2018,
        diagnosis_maps.Firdos_2016,
        diagnosis_maps.Kim_2024,
        diagnosis_maps.Sztaho_2018,
        diagnosis_maps.Tsui_2018,
        diagnosis_maps.USVAC_2025,
        diagnosis_maps.Zaim_2023,
    ]

    with open(output_path, "w") as output_file:
        output_file.write("# DB Sessions per classification system\n")
        output_file.write(
            "Count of sessions for different databases for different classification systems at all available levels of classification in the classification system.\n"
        )
        for diag_map in tqdm(diag_maps, desc="counting per diag_map", position=0):
            diag_map = diag_map()
            output_file.write(f"## {diag_map.name}\n")
            for db_name in tqdm(db_names, desc="counting per db", position=1):
                db = await generator.count_for_diag_map(
                    db_name=db_name,
                    source_path=data_store_path,
                    diag_map=diag_map,
                )
                output_file.write(f"### {db_name}\n")
                for level in range(diag_map.max_diag_level + 1):
                    counts = db.count_per_diag(level=level)
                    output_file.write(f"* **Level {level}:**\n")
                    for diag, speaker_ids in counts.items():
                        count = len(speaker_ids)
                        output_file.write(f"\t * **{diag.name}:** {count}\n")



import yaml
import typing
from pathlib import Path
from typing import Literal
from .generator import DatabaseFunc, Dataset
from divr_diagnosis import DiagnosisMap

from .GeneratorV1 import GeneratorV1

VERSIONS = Literal["v1"]
versions = typing.get_args(VERSIONS)
generator_map = {"v1": GeneratorV1()}


class YamlDumper(yaml.Dumper):
    def increase_indent(self, flow=False, indentless=False):
        return super(YamlDumper, self).increase_indent(flow, False)


async def generate_tasks(
    version: VERSIONS, source_path: Path, diagnosis_map: DiagnosisMap
) -> None:
    module_path = Path(__file__).parent.parent.resolve()
    tasks_path = Path(f"{module_path}/tasks/{version}")
    await generator_map[version](
        source_path=source_path,
        tasks_path=tasks_path,
        diagnosis_map=diagnosis_map,
    )


async def collect_diagnosis_terms(version: VERSIONS, source_path: Path):
    output_file_path = f"{source_path}/diag_terms.yml"
    terms = await generator_map[version].collect_diagnosis_terms(
        source_path=source_path
    )
    terms = {key: list(val) for key, val in terms.items()}
    with open(output_file_path, "w") as output_file:
        yaml.dump(terms, output_file, Dumper=YamlDumper, allow_unicode=True)
    print(f"terms collected into {output_file_path}")



from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class UASpeech(Base):
    DB_NAME = "uaspeech"
    __df = [
        {"id": "CF02", "diagnosis": "normal", "gender": "F", "age": None},
        {"id": "CF03", "diagnosis": "normal", "gender": "F", "age": None},
        {"id": "CF04", "diagnosis": "normal", "gender": "F", "age": None},
        {"id": "CF05", "diagnosis": "normal", "gender": "F", "age": None},
        {"id": "CM01", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM04", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM05", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM06", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM08", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM09", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM10", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM12", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "CM13", "diagnosis": "normal", "gender": "M", "age": None},
        {"id": "M01", "diagnosis": "Spastic", "gender": "M", "age": "18"},
        {"id": "M04", "diagnosis": "Spastic", "gender": "M", "age": "18"},
        {"id": "M05", "diagnosis": "Spastic", "gender": "M", "age": "21"},
        {"id": "M06", "diagnosis": "Spastic", "gender": "M", "age": "18"},
        {"id": "M07", "diagnosis": "Spastic", "gender": "M", "age": "58"},
        {"id": "M08", "diagnosis": "Spastic", "gender": "M", "age": "28"},
        {"id": "M09", "diagnosis": "Spastic", "gender": "M", "age": "18"},
        {"id": "M10", "diagnosis": "Not sure", "gender": "M", "age": "21"},
        {"id": "F02", "diagnosis": "Spastic", "gender": "F", "age": "30"},
        {"id": "F03", "diagnosis": "Spastic", "gender": "F", "age": "51"},
        {
            "id": "F04",
            "diagnosis": "Athetoid (or mixed)",
            "gender": "F",
            "age": "18",
        },
        {"id": "F05", "diagnosis": "Spastic", "gender": "F", "age": "22"},
        {"id": "M11", "diagnosis": "Athetoid", "gender": "M", "age": "48"},
        {"id": "M12", "diagnosis": "Mixed", "gender": "M", "age": "19"},
        {"id": "M13", "diagnosis": "Spastic", "gender": "M", "age": "44"},
        {"id": "M14", "diagnosis": "Spastic", "gender": "M", "age": "40"},
        {"id": "M16", "diagnosis": "Spastic", "gender": "M", "age": None},
    ]

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        return set([data["diagnosis"].lower() for data in self.__df])

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        data_path = f"{source_path}/UASpeech/audio/original/"

        for data in self.__df:
            speaker_id = data["id"]
            diagnosis = data["diagnosis"].lower()
            diagnosis = (
                diagnosis_map[diagnosis]
                if diagnosis in diagnosis_map
                else diagnosis_map.unclassified
            )
            if allow_incomplete_classification or not diagnosis.incompletely_classified:
                age = int(data["age"]) if data["age"] is not None else None
                gender = Gender.format(data["gender"].strip())
                file_paths = [
                    path
                    for path in Path(f"{data_path}/{speaker_id}").glob("*.wav")
                    if not path.name.startswith(".")
                ]
                num_files = len(file_paths)
                if min_tasks is None or num_files >= min_tasks:
                    session = ProcessedSession(
                        id=f"uaspeech_{speaker_id}",
                        speaker_id=speaker_id,
                        age=age,
                        gender=gender,
                        diagnosis=[diagnosis],
                        files=[ProcessedFile(path=path) for path in file_paths],
                        num_files=num_files,
                    )
                    sessions += [session]
        return sessions



import pandas as pd
from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class Voiced(Base):
    DB_NAME = "voiced"

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        _, df = self.__read_data(source_path=source_path)
        return set(df["Diagnosis"].tolist())

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        data_path, all_data = self.__read_data(source_path)

        for _, row in all_data.iterrows():
            speaker_id = row["ID"]
            diagnosis = row["Diagnosis"]
            diagnosis = (
                diagnosis_map[diagnosis]
                if diagnosis in diagnosis_map
                else diagnosis_map.unclassified
            )
            age = int(row["Age"])
            gender = Gender.format(row["Gender"])
            if allow_incomplete_classification or not diagnosis.incompletely_classified:
                num_files = 1
                if min_tasks is None or num_files >= min_tasks:
                    sessions += [
                        ProcessedSession(
                            id=f"voiced_{speaker_id}",
                            speaker_id=speaker_id,
                            age=age,
                            gender=gender,
                            diagnosis=[diagnosis],
                            files=[
                                ProcessedFile(
                                    path=Path(f"{data_path}/{speaker_id}.wav")
                                )
                            ],
                            num_files=num_files,
                        )
                    ]
        return sessions

    def __read_data(self, source_path):
        data_path = f"{source_path}/voice-icar-federico-ii-database-1.0.0"

        info_files = list(Path(data_path).rglob("*-info.txt"))
        rows = []
        for ifile in info_files:
            df = pd.read_csv(ifile, delimiter="\t", header=None)
            df.dropna(how="all", inplace=True)
            row = pd.Series(
                list(df[1]), index=df[0].apply(lambda x: x.replace(":", ""))
            )
            row = self.__fix_errors(ifile, row)
            rows += [row]
        all_data = pd.DataFrame(rows)
        all_data = all_data[["ID", "Diagnosis", "Gender", "Age"]]
        all_data["Diagnosis"] = all_data["Diagnosis"].str.lower().str.strip()
        return data_path, all_data

    def __fix_errors(self, ifile: Path, row: pd.Series) -> pd.Series:
        """
        Used for fixing errors in the DB
        """
        filekey = ifile.stem.removesuffix("-info")
        if row["ID"] != filekey:
            print(f"Info: Fixing DB error where original ID={row['ID']}, ifile={ifile}")
            row["ID"] = filekey
        return row



import asyncio
from pathlib import Path
from divr_diagnosis import DiagnosisMap
from typing import Awaitable, Callable, Dict, List

from .generator import Generator, DatabaseFunc, Dataset
from .databases import (
    SVD,
    Torgo,
    Voiced,
    AVFAD,
    MEEI,
    UASpeech,
    UncommonVoice,
    Base as Database,
)
from .databases.svd import VOWELS


class GeneratorV1(Generator):

    __db_map = {
        AVFAD.DB_NAME: AVFAD,
        MEEI.DB_NAME: MEEI,
        SVD.DB_NAME: SVD,
        Torgo.DB_NAME: Torgo,
        UASpeech.DB_NAME: UASpeech,
        UncommonVoice.DB_NAME: UncommonVoice,
        Voiced.DB_NAME: Voiced,
    }

    async def collect_diagnosis_terms(self, source_path: Path) -> Dict[str, List[str]]:
        dbs = [AVFAD, MEEI, SVD, Torgo, UASpeech, UncommonVoice, Voiced]
        terms = {}
        for db in dbs:
            for term in await db(source_path=source_path).collect_diagnosis_terms():
                if term not in terms:
                    terms[term] = set()
                terms[term].add(db.DB_NAME)
        del terms[""]
        return terms

    async def count_for_diag_map(
        self, db_name: str, source_path: Path, diag_map: DiagnosisMap
    ) -> Database:
        db = self.__db_map[db_name](source_path=source_path)
        await db.init(
            diagnosis_map=diag_map,
            allow_incomplete_classification=True,
            min_tasks=None,
        )
        return db

    async def generate_task(
        self,
        source_path: Path,
        filter_func: Callable[[DatabaseFunc], Awaitable[Dataset]],
        task_path: Path,
        diagnosis_map: DiagnosisMap,
        allow_incomplete_classification: bool = False,
    ) -> None:
        async def __database(name: str, min_tasks: int | None = None) -> Database:
            db = self.__db_map[name.lower()](source_path=source_path)
            await db.init(
                diagnosis_map=diagnosis_map,
                allow_incomplete_classification=allow_incomplete_classification,
                min_tasks=min_tasks,
            )
            return db

        tasks = await filter_func(__database)
        self.to_task_file(tasks=tasks.train, output_path=Path(f"{task_path}/train"))
        self.to_task_file(tasks=tasks.val, output_path=Path(f"{task_path}/val"))
        self.to_task_file(tasks=tasks.test, output_path=Path(f"{task_path}/test"))

    async def __call__(
        self,
        source_path: Path,
        tasks_path: Path,
        diagnosis_map: DiagnosisMap,
    ) -> None:
        print("Generating benchmark v1 tasks")
        Path(f"{tasks_path}/streams").mkdir(exist_ok=True)
        svd = SVD(source_path=source_path)
        torgo = Torgo(source_path=source_path)
        voiced = Voiced(source_path=source_path)
        await asyncio.gather(
            svd.init(
                diagnosis_map=diagnosis_map,
                allow_incomplete_classification=False,
                min_tasks=SVD.max_tasks,
            ),
            torgo.init(
                diagnosis_map=diagnosis_map,
                allow_incomplete_classification=False,
                min_tasks=None,
            ),
            voiced.init(
                diagnosis_map=diagnosis_map,
                allow_incomplete_classification=False,
                min_tasks=None,
            ),
        )
        self.__stream0(
            stream_path=Path(f"{tasks_path}/streams/0"),
            svd=svd,
            torgo=torgo,
            voiced=voiced,
        )
        self.__stream1(
            stream_path=Path(f"{tasks_path}/streams/1"),
            svd=svd,
        )
        self.__stream2(
            stream_path=Path(f"{tasks_path}/streams/2"),
            svd=svd,
            torgo=torgo,
            voiced=voiced,
        )
        self.__stream3(
            stream_path=Path(f"{tasks_path}/streams/3"),
            svd=svd,
            torgo=torgo,
            voiced=voiced,
        )

    def __stream0(self, stream_path: Path, svd: SVD, voiced: Voiced, torgo: Torgo):
        print(f"Generating stream 0 from at {stream_path}")
        stream_path.mkdir(exist_ok=True)
        self.to_task_file(
            (
                svd.all_train(level=0)
                + torgo.all_train(level=0)
                + voiced.all_train(level=0)
            ),
            Path(f"{stream_path}/train"),
        )
        self.to_task_file(
            (svd.all_val(level=0) + torgo.all_val(level=0) + voiced.all_val(level=0)),
            Path(f"{stream_path}/val"),
        )

        # task 1
        Path(f"{stream_path}/{1}").mkdir(exist_ok=True)
        self.to_task_file(
            svd.test_set_neutral_vowels(level=0),
            Path(f"{stream_path}/{1}/test"),
        )

        # task 2
        Path(f"{stream_path}/{2}").mkdir(exist_ok=True)
        self.to_task_file(
            svd.test_set_connected_speech(level=0),
            Path(f"{stream_path}/{2}/test"),
        )

        # task 3
        Path(f"{stream_path}/{3}").mkdir(exist_ok=True)
        self.to_task_file(
            voiced.all_test(level=0),
            Path(f"{stream_path}/{3}/test"),
        )

        # task 4
        Path(f"{stream_path}/{4}").mkdir(exist_ok=True)
        self.to_task_file(
            torgo.all_test(level=0),
            Path(f"{stream_path}/{4}/test"),
        )

    def __stream1(self, stream_path: Path, svd: SVD):
        print(f"Generating stream 1 from at {stream_path}")
        stream_path.mkdir(exist_ok=True)

        def single_vowel_task(task_idx: int, level: int, vowel: VOWELS):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                svd.train_set_neutral_vowels(level=level, vowel=vowel),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.test_set_neutral_vowels(level=level, vowel=vowel),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.val_set_neutral_vowels(level=level, vowel=vowel),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def connected_speech_task(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                svd.train_set_connected_speech(level=level),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.test_set_connected_speech(level=level),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.val_set_connected_speech(level=level),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def combined_vocalisation_task(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                svd.train_set_combined_vowel_vocalisation(level=level),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.test_set_combined_vowel_vocalisation(level=level),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.val_set_combined_vowel_vocalisation(level=level),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def multi_vowel_task(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            vowels: List[VOWELS] = ["a", "i", "u"]
            self.to_task_file(
                svd.train_set_multi_neutral_vowels(level=level, vowels=vowels),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.test_set_multi_neutral_vowels(level=level, vowels=vowels),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.val_set_multi_neutral_vowels(level=level, vowels=vowels),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def lhl_vowel_task(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                svd.train_set_lhl_vowels(level=level, vowel="a"),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.test_set_lhl_vowels(level=level, vowel="a"),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.val_set_lhl_vowels(level=level, vowel="a"),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def all_combined_task(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            suffixes = [
                "a_n.wav",
                "i_n.wav",
                "u_n.wav",
                "a_lhl.wav",
                "i_lhl.wav",
                "u_lhl.wav",
                "iau.wav",
                "-phrase.wav",
            ]
            self.to_task_file(
                svd.filtered_multi_file_tasks(
                    sessions=svd.dataset.train_sessions, level=level, suffixes=suffixes
                ),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.filtered_multi_file_tasks(
                    sessions=svd.dataset.test_sessions, level=level, suffixes=suffixes
                ),
                Path(f"{stream_path}/{task_idx}/test"),
            )
            self.to_task_file(
                svd.filtered_multi_file_tasks(
                    sessions=svd.dataset.val_sessions, level=level, suffixes=suffixes
                ),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        # task 1
        single_vowel_task(task_idx=1, level=1, vowel="a")
        # task 2
        single_vowel_task(task_idx=2, level=1, vowel="i")
        # task 3
        single_vowel_task(task_idx=3, level=1, vowel="u")
        # task 4
        lhl_vowel_task(task_idx=4, level=1)
        # task 5
        multi_vowel_task(task_idx=5, level=1)
        # task 6
        combined_vocalisation_task(task_idx=6, level=1)
        # task 7
        connected_speech_task(task_idx=7, level=1)
        # task 8
        all_combined_task(task_idx=8, level=1)
        # task 9
        single_vowel_task(task_idx=9, level=2, vowel="a")
        # task 10
        single_vowel_task(task_idx=10, level=2, vowel="i")
        # task 11
        single_vowel_task(task_idx=11, level=2, vowel="u")
        # task 12
        lhl_vowel_task(task_idx=12, level=2)
        # task 13
        multi_vowel_task(task_idx=13, level=2)
        # task 14
        combined_vocalisation_task(task_idx=14, level=2)
        # task 15
        connected_speech_task(task_idx=15, level=2)
        # task 16
        all_combined_task(task_idx=16, level=2)

    def __stream2(self, stream_path: Path, svd: SVD, torgo: Torgo, voiced: Voiced):
        print(f"Generating stream 2 from at {stream_path}")
        stream_path.mkdir(exist_ok=True)

        def train_val_set(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                (
                    svd.train_set_neutral_vowels(level=level, vowel="a")
                    + svd.test_set_neutral_vowels(level=level, vowel="a")
                ),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                svd.val_set_neutral_vowels(level=level, vowel="a"),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        def voiced_test_set(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                (
                    voiced.all_test(level=level)
                    + voiced.all_train(level=level)
                    + voiced.all_val(level=level)
                ),
                Path(f"{stream_path}/{task_idx}/test"),
            )

        def torgo_test_set(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                (
                    torgo.all_test(level=level)
                    + torgo.all_train(level=level)
                    + torgo.all_val(level=level)
                ),
                Path(f"{stream_path}/{task_idx}/test"),
            )

        # task 1
        train_val_set(task_idx=1, level=0)
        voiced_test_set(task_idx=1, level=0)
        # task 2
        train_val_set(task_idx=2, level=1)
        voiced_test_set(task_idx=2, level=1)
        # task 3
        train_val_set(task_idx=3, level=2)
        voiced_test_set(task_idx=3, level=2)
        # task 4
        train_val_set(task_idx=4, level=0)
        torgo_test_set(task_idx=4, level=0)

    def __stream3(self, stream_path: Path, svd: SVD, torgo: Torgo, voiced: Voiced):
        print(f"Generating stream 3 from at {stream_path}")
        stream_path.mkdir(exist_ok=True)

        def train_val_set(task_idx: int, level: int):
            Path(f"{stream_path}/{task_idx}").mkdir(exist_ok=True)
            self.to_task_file(
                (voiced.all_train(level=level) + torgo.all_train(level=level)),
                Path(f"{stream_path}/{task_idx}/train"),
            )
            self.to_task_file(
                (voiced.all_val(level=level) + torgo.all_val(level=level)),
                Path(f"{stream_path}/{task_idx}/val"),
            )

        # task 1
        train_val_set(task_idx=1, level=1)
        self.to_task_file(
            voiced.all_test(level=1),
            Path(f"{stream_path}/{1}/test"),
        )
        # task 2
        train_val_set(task_idx=2, level=0)
        self.to_task_file(
            torgo.all_test(level=0),
            Path(f"{stream_path}/{2}/test"),
        )
        # task 3
        train_val_set(task_idx=3, level=1)
        self.to_task_file(
            (
                svd.test_set_neutral_vowels(level=1, vowel="a")
                + svd.val_set_neutral_vowels(level=1, vowel="a")
                + svd.train_set_neutral_vowels(level=1, vowel="a")
            ),
            Path(f"{stream_path}/{3}/test"),
        )
        # task 4
        train_val_set(task_idx=4, level=2)
        self.to_task_file(
            voiced.all_test(level=2),
            Path(f"{stream_path}/{4}/test"),
        )
        # task 5
        train_val_set(task_idx=5, level=2)
        self.to_task_file(
            (
                svd.test_set_neutral_vowels(level=2, vowel="a")
                + svd.val_set_neutral_vowels(level=2, vowel="a")
                + svd.train_set_neutral_vowels(level=2, vowel="a")
            ),
            Path(f"{stream_path}/{5}/test"),
        )



from pathlib import Path
from typing import Callable, Dict, List, Set
from divr_diagnosis import Diagnosis, DiagnosisMap

from ...prepare_dataset.database_generator import DatabaseGenerator
from ...prepare_dataset.processed import (
    ProcessedFile,
    ProcessedSession,
)
from ..task import Task

FileFilter = Callable[[List[ProcessedFile]], List[ProcessedFile]]


class Base:
    DB_NAME: str

    def __init__(
        self,
        source_path: Path,
    ) -> None:
        self.__source_path = source_path
        self.__db_source_path = Path(f"{source_path}/{self.DB_NAME}")

    async def init(
        self,
        diagnosis_map: DiagnosisMap,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
    ):
        database_generator = DatabaseGenerator(
            train_split=0.7,
            test_split=0.2,
            random_seed=42,
        )
        sessions = await self.prepare_dataset(
            source_path=self.__db_source_path,
            allow_incomplete_classification=allow_incomplete_classification,
            min_tasks=min_tasks,
            diagnosis_map=diagnosis_map,
        )
        self.dataset = database_generator.generate(
            db_name=self.DB_NAME,
            sessions=sessions,
        )

    async def collect_diagnosis_terms(self) -> Set[str]:
        return await self._collect_diagnosis_terms(source_path=self.__db_source_path)

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        raise NotImplementedError()

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        raise NotImplementedError()

    def to_audio_key(self, source_path: ProcessedFile) -> str:
        return (
            str(source_path.path)
            .removeprefix(str(self.__source_path))
            .removeprefix("/")
        )

    def all_train(self, level: int) -> List[Task]:
        return self.to_individual_file_tasks(
            self.dataset.train_sessions, level=level, file_filter=None
        )

    def all_val(self, level: int) -> List[Task]:
        return self.to_individual_file_tasks(
            self.dataset.val_sessions, level=level, file_filter=None
        )

    def all_test(self, level: int) -> List[Task]:
        return self.to_individual_file_tasks(
            self.dataset.test_sessions, level=level, file_filter=None
        )

    def all(self, level: int) -> List[Task]:
        return self.to_individual_file_tasks(
            sessions=self.dataset.all_sessions, level=level, file_filter=None
        )

    def count_per_diag(self, level: int) -> Dict[Diagnosis, set[str]]:
        counts: dict[Diagnosis, set[str]] = {}
        for session in self.dataset.all_sessions:
            root_diagnosis = session.best_diagnosis.at_level(level)
            if root_diagnosis not in counts:
                counts[root_diagnosis] = set()
            counts[root_diagnosis].add(session.speaker_id)
        return counts

    def to_individual_file_tasks(
        self,
        sessions: List[ProcessedSession],
        level: int,
        file_filter: FileFilter | None,
    ) -> List[Task]:
        tasks: List[Task] = []
        for session in sessions:
            root_diagnosis = session.best_diagnosis.at_level(level)
            if file_filter is None:
                files = session.files
            else:
                files = file_filter(session.files)
            for file_idx, file_path in enumerate(files):
                task = Task(
                    id=f"{session.id}_{file_idx}",
                    speaker_id=session.speaker_id,
                    age=session.age,
                    gender=session.gender,
                    label=root_diagnosis,
                    audio_keys=[self.to_audio_key(file_path)],
                )
                tasks.append(task)
        return tasks

    def to_multi_file_tasks(
        self,
        sessions: List[ProcessedSession],
        level: int,
        file_filter: FileFilter | None,
    ) -> List[Task]:
        tasks: List[Task] = []
        for session in sessions:
            root_diagnosis = session.best_diagnosis.at_level(level)
            if file_filter is None:
                files = session.files
            else:
                files = file_filter(session.files)
            task = Task(
                id=f"{session.id}",
                speaker_id=session.speaker_id,
                age=session.age,
                gender=session.gender,
                label=root_diagnosis,
                audio_keys=list(map(self.to_audio_key, files)),
            )
            tasks.append(task)
        return tasks



import os
from setuptools import setup, find_packages

with open("README.md", "r") as readme:
    long_description = readme.read()

setup(
    name="divr-benchmark",
    packages=find_packages(),
    package_data={
        "divr_benchmark": ["../tasks/*/*.yml", "../tasks/README.md"],
    },
    version=os.environ["RELEASE_VERSION"],
    license="MIT",
    description="Toolkit to work with disordered voice databases",
    author="Computational Audio Research Lab",
    url="https://github.com/ComputationalAudioResearchLab/divr-benchmark",
    keywords=[
        "ML Audio Features",
        "ML",
        "Disordered Voice Features",
        "Research",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=[
        "aiofiles>=23.1.0",
        "aiohttp>=3.8.4",
        "class-argparse>=0.1.3",
        "divr-diagnosis>=0.1.2",
        "librosa>=0.10.0.post2",
        "matplotlib>=3.7.1",
        "nspfile>=0.1.4",
        "openpyxl>=3.1.2",
        "pandas>=2.0.1",
        "PyYAML>=6.0.1",
        "svd-downloader>=0.1.1",
        "wfdb>=4.1.2",
        "xlrd>=2.0.1",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
)



import pandas as pd
from pathlib import Path
from typing import List, Set
from divr_diagnosis import DiagnosisMap

from .Base import Base
from .gender import Gender
from ...prepare_dataset.processed import (
    ProcessedSession,
    ProcessedFile,
)


class MEEI(Base):
    DB_NAME = "meei"

    async def _collect_diagnosis_terms(self, source_path: Path) -> Set[str]:
        file_key, full_data = self.__read_data(source_path)
        diags = full_data["DIAGNOSIS"].str.split(",").explode().str.lower().str.strip()
        return set(diags)

    async def prepare_dataset(
        self,
        source_path: Path,
        allow_incomplete_classification: bool,
        min_tasks: int | None,
        diagnosis_map: DiagnosisMap,
    ) -> List[ProcessedSession]:
        sessions = []
        audio_extraction_path = Path(f"{source_path}/.extracted")
        audio_extraction_path.mkdir(exist_ok=True)
        file_key, full_data = self.__read_data(source_path)
        for _, row in full_data.iterrows():
            speaker_id = row[file_key][:5]
            diagnosis = []
            has_incomplete_diagnosis = False
            for x in row["DIAGNOSIS"].split(","):
                x = x.lower().strip()
                if len(x) > 0:
                    d = (
                        diagnosis_map[x]
                        if x in diagnosis_map
                        else diagnosis_map.unclassified
                    )
                    diagnosis += [d]
                    if d.incompletely_classified:
                        has_incomplete_diagnosis = True
            if len(diagnosis) == 0:
                diagnosis = [diagnosis_map.get("unknown")]
                has_incomplete_diagnosis = True
            if allow_incomplete_classification or not has_incomplete_diagnosis:
                age = int(row["AGE"]) if row["AGE"] != "" else None
                gender = Gender.format(row["SEX"].strip())
                file_paths = list(Path(source_path).rglob(f"{speaker_id}*.NSP"))
                num_files = len(file_paths)
                if min_tasks is None or num_files >= min_tasks:
                    sessions += [
                        ProcessedSession(
                            id=f"meei_{speaker_id}",
                            speaker_id=speaker_id,
                            age=age,
                            gender=gender,
                            diagnosis=diagnosis,
                            files=[
                                await ProcessedFile.from_nsp(
                                    nsp_path=path,
                                    extraction_path=audio_extraction_path,
                                )
                                for path in file_paths
                            ],
                            num_files=num_files,
                        )
                    ]
        return sessions

    def __read_data(self, source_path):
        excel_path = f"{source_path}/kaylab/data/disorderedvoicedb/EXCEL50/KAYCD_DB.XLS"
        file_key = "FILE VOWEL 'AH'"

        df_path = self.__clean_white_spaces(
            pd.read_excel(
                excel_path,
                sheet_name="Pathological",
                nrows=654,
            )
        )
        df_norm = self.__clean_white_spaces(
            pd.read_excel(excel_path, sheet_name="Normal", nrows=53)
        )
        df_full = self.__clean_white_spaces(
            pd.read_excel(excel_path, sheet_name="Full Database")
        )
        path_files = df_path[file_key]
        norm_files = df_norm[file_key]

        path_data = self.__collate_data(
            df=df_full[df_full[file_key].isin(path_files)],
            is_pathological=True,
            file_key=file_key,
        )
        norm_data = self.__collate_data(
            df=df_full[df_full[file_key].isin(norm_files)],
            is_pathological=False,
            file_key=file_key,
        )
        full_data = pd.concat([path_data, norm_data])
        full_data = full_data.sort_values(by=full_data.columns.tolist())
        return file_key, full_data

    def __collate_data(self, df: pd.DataFrame, is_pathological: bool, file_key: str):
        grouped = df.groupby(file_key)
        sex = grouped["SEX"].apply(lambda x: set(x).pop() if len(x) > 0 else None)
        age = grouped["AGE"].apply(lambda x: set(x).pop() if len(x) > 0 else None)
        diagnosis = grouped["DIAGNOSIS"].apply(lambda x: ",".join(set(x)))
        df = pd.concat([diagnosis, sex, age], axis=1)
        df = df.reset_index()
        df["pathological"] = is_pathological
        return df

    def __clean_white_spaces(self, df: pd.DataFrame):
        clean_columns = list(map(str.strip, df.columns))
        df.columns = clean_columns
        df = df.map(lambda x: x.strip() if isinstance(x, str) else x)
        return df



from .Base import Base
from .svd import SVD
from .torgo import Torgo
from .voiced import Voiced
from .avfad import AVFAD
from .meei import MEEI
from .uaspeech import UASpeech
from .uncommon_voice import UncommonVoice

__all__ = [
    "Base",
    "SVD",
    "Torgo",
    "Voiced",
    "AVFAD",
    "MEEI",
    "UASpeech",
    "UncommonVoice",
]



from typing import List
from dataclasses import dataclass
from divr_diagnosis import Diagnosis


@dataclass
class Task:
    id: str
    speaker_id: str
    age: int | None
    gender: str
    label: Diagnosis
    audio_keys: List[str]
